==> rest-sdk-commons-0.8.0-sources.jar: com/mulesoft/connectivity/rest/commons/internal/model/dataexpressions/SequenceCompositeDataExpression.java <==
/*
 * (c) 2003-2021 MuleSoft, Inc. This software is protected under international copyright
 * law. All use of this software is subject to MuleSoft's Master Subscription Agreement
 * (or other master license agreement) separately entered into in writing between you and
 * MuleSoft. If such an agreement is not in place, you may not use the software.
 */
package com.mulesoft.connectivity.rest.commons.internal.model.dataexpressions;

import static com.mulesoft.connectivity.rest.commons.internal.RestConstants.ATTRIBUTES_VAR;
import static com.mulesoft.connectivity.rest.commons.internal.RestConstants.PAYLOAD_VAR;
import static java.lang.String.format;
import static org.mule.runtime.api.i18n.I18nMessageFactory.createStaticMessage;

import org.mule.runtime.api.exception.MuleRuntimeException;
import org.mule.runtime.api.metadata.DataType;
import org.mule.runtime.api.metadata.MediaType;
import org.mule.runtime.api.metadata.TypedValue;
import org.mule.runtime.extension.api.runtime.operation.Result;

import com.mulesoft.connectivity.rest.commons.internal.model.common.EvaluationContext;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * A data expression that composes multiple data expressions in steps. A step will always have access to the result of the
 * previous step. If the defined steps are named, further steps will also have access to its result by this name.
 *
 * @since 1.0
 */
public class SequenceCompositeDataExpression implements DataExpression {

  public static class Step {

    private final String name;
    private final DataExpression dataExpression;

    public Step(String name, DataExpression dataExpression) {
      this.name = name;
      this.dataExpression = dataExpression;
    }

    public Step(DataExpression dataExpression) {
      this(null, dataExpression);
    }

    public Optional<String> getName() {
      return Optional.ofNullable(name);
    }

    public DataExpression getDataExpression() {
      return dataExpression;
    }
  }

  private final List<Step> steps = new ArrayList<>();

  public SequenceCompositeDataExpression(List<Step> steps) {
    this.steps.addAll(steps);
  }

  @Override
  public Object evaluate(EvaluationContext evaluationContext) {
    for (Step step : steps) {
      Object result = step.getDataExpression().evaluate(evaluationContext);
      // TODO RSDK-262: Is unwrapping here ok?
      defineResultForNextEvaluationContext(evaluationContext, result);
      defineStepForNextEvaluationContext(evaluationContext, step, result);
    }

    return collectResultFromEvaluationContext(evaluationContext);
  }

  // TODO RSDK-262: Could be simplified and merged with #defineResultForNextEvaluationContext
  private void defineStepForNextEvaluationContext(EvaluationContext evaluationContext, Step step, Object result) {
    if (step.getName().isPresent()) {
      final Map<String, Object> stepResult = new HashMap<>();

      if (result instanceof Result<?, ?>) {
        Result<?, ?> resultResult = (Result<?, ?>) result;
        stepResult.put(PAYLOAD_VAR, resultResult.getOutput());
        stepResult.put(ATTRIBUTES_VAR, new TypedValue<>(resultResult.getAttributes(), DataType.builder()
            .mediaType(resultResult.getAttributesMediaType().orElse(MediaType.ANY)).build()));
      } else {
        stepResult.put(PAYLOAD_VAR, result);
      }

      evaluationContext.define(step.getName().get(), TypedValue.of(stepResult));
    }
  }

  private void defineResultForNextEvaluationContext(EvaluationContext evaluationContext, Object result) {
    if (result instanceof Result<?, ?>) {
      Result<?, ?> resultResult = (Result<?, ?>) result;
      evaluationContext.define(PAYLOAD_VAR, resultResult.getOutput());
      evaluationContext.define(ATTRIBUTES_VAR, new TypedValue<>(resultResult.getAttributes(), DataType.builder()
          .mediaType(resultResult.getAttributesMediaType().orElse(MediaType.ANY)).build()));
    } else {
      evaluationContext.define(PAYLOAD_VAR, result);
    }
  }

  private Result collectResultFromEvaluationContext(EvaluationContext evaluationContext) {
    TypedValue tOutput = getTypedValue(evaluationContext, PAYLOAD_VAR);
    TypedValue tOutputAttributes = getTypedValue(evaluationContext, ATTRIBUTES_VAR);

    Result.Builder builder = Result.builder();

    if (tOutputAttributes != null) {
      ((Optional) tOutputAttributes.getValue()).ifPresent(attributes -> builder.attributes(attributes));
      builder.attributesMediaType(tOutputAttributes.getDataType().getMediaType());
    }

    return builder
        .output(tOutput)
        .mediaType(tOutput.getDataType().getMediaType())
        .build();
  }

  private TypedValue getTypedValue(EvaluationContext evaluationContext, String variableName) {
    Object variable = evaluationContext.getVariables().get(variableName);
    if (variable == null) {
      return null;
    }
    if (!(variable instanceof TypedValue)) {
      throw new MuleRuntimeException(createStaticMessage(
                                                         format("'%s' variable from evaluation context should be a '%s', instead it is a '%s'. This is a probably a bug.",
                                                                variableName, TypedValue.class.getName(),
                                                                variable.getClass().getName())));
    }
    return (TypedValue) variable;
  }

}

==> rest-sdk-commons-0.11.0-sources.jar: com/mulesoft/connectivity/rest/commons/internal/model/dataexpressions/SequenceCompositeDataExpression.java <==
/*
 * (c) 2003-2021 MuleSoft, Inc. This software is protected under international copyright
 * law. All use of this software is subject to MuleSoft's Master Subscription Agreement
 * (or other master license agreement) separately entered into in writing between you and
 * MuleSoft. If such an agreement is not in place, you may not use the software.
 */
package com.mulesoft.connectivity.rest.commons.internal.model.dataexpressions;

import static com.mulesoft.connectivity.rest.commons.internal.RestConstants.ATTRIBUTES_VAR;
import static com.mulesoft.connectivity.rest.commons.internal.RestConstants.PAYLOAD_VAR;
import static java.lang.String.format;
import static org.mule.runtime.api.i18n.I18nMessageFactory.createStaticMessage;

import org.mule.runtime.api.exception.MuleRuntimeException;
import org.mule.runtime.api.metadata.DataType;
import org.mule.runtime.api.metadata.MediaType;
import org.mule.runtime.api.metadata.TypedValue;
import org.mule.runtime.extension.api.runtime.operation.Result;

import com.mulesoft.connectivity.rest.commons.api.dw.DWBindings;
import com.mulesoft.connectivity.rest.commons.api.dw.HttpResponseDWBinding;
import com.mulesoft.connectivity.rest.commons.api.operation.HttpResponseAttributes;
import com.mulesoft.connectivity.rest.commons.internal.model.common.EvaluationContext;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * A data expression that composes multiple data expressions in steps. A step will always have access to the result of the
 * previous step. If the defined steps are named, further steps will also have access to its result by this name.
 *
 * @since 1.0
 */
public class SequenceCompositeDataExpression implements DataExpression {

  public static class Step {

    private final String name;
    private final DataExpression dataExpression;

    public Step(String name, DataExpression dataExpression) {
      this.name = name;
      this.dataExpression = dataExpression;
    }

    public Step(DataExpression dataExpression) {
      this(null, dataExpression);
    }

    public Optional<String> getName() {
      return Optional.ofNullable(name);
    }

    public DataExpression getDataExpression() {
      return dataExpression;
    }
  }

  private final List<Step> steps = new ArrayList<>();

  public SequenceCompositeDataExpression(List<Step> steps) {
    this.steps.addAll(steps);
  }

  @Override
  public Object evaluate(EvaluationContext evaluationContext) {
    for (Step step : steps) {
      Object result = step.getDataExpression().evaluate(evaluationContext);
      // TODO RSDK-262: Is unwrapping here ok?
      defineResultForNextEvaluationContext(evaluationContext, result);
      defineStepForNextEvaluationContext(evaluationContext, step, result);
    }

    return collectResultFromEvaluationContext(evaluationContext);
  }

  // TODO RSDK-262: Could be simplified and merged with #defineResultForNextEvaluationContext
  private void defineStepForNextEvaluationContext(EvaluationContext evaluationContext, Step step, Object result) {
    if (step.getName().isPresent()) {
      final Map<String, Object> stepResult = new HashMap<>();

      if (result instanceof Result<?, ?>) {
        Result<TypedValue<String>, HttpResponseAttributes> resultResult =
            (Result<TypedValue<String>, HttpResponseAttributes>) result;
        HttpResponseAttributes attributes = resultResult.getAttributes().orElse(null);
        HttpResponseDWBinding responseDWBinding = new HttpResponseDWBinding(resultResult.getOutput(), attributes);
        stepResult.put(DWBindings.RESPONSE.getBinding(), responseDWBinding);
        stepResult.put(PAYLOAD_VAR, resultResult.getOutput());
        stepResult.put(ATTRIBUTES_VAR, new TypedValue<>(resultResult.getAttributes(), DataType.builder()
            .mediaType(resultResult.getAttributesMediaType().orElse(MediaType.ANY)).build()));
      } else {
        TypedValue<String> body = ((TypedValue<String>) result);
        HttpResponseDWBinding responseDWBinding = new HttpResponseDWBinding(body, null);
        evaluationContext.define(DWBindings.RESPONSE.getBinding(), responseDWBinding);
        stepResult.put(PAYLOAD_VAR, result);
      }

      evaluationContext.define(step.getName().get(), TypedValue.of(stepResult));
    }
  }

  private void defineResultForNextEvaluationContext(EvaluationContext evaluationContext, Object result) {
    if (result instanceof Result<?, ?>) {
      Result<TypedValue<String>, HttpResponseAttributes> resultResult =
          (Result<TypedValue<String>, HttpResponseAttributes>) result;
      HttpResponseAttributes attributes = resultResult.getAttributes().orElse(null);
      HttpResponseDWBinding responseDWBinding = new HttpResponseDWBinding(resultResult.getOutput(), attributes);
      evaluationContext.define(DWBindings.RESPONSE.getBinding(), responseDWBinding);
      evaluationContext.define(PAYLOAD_VAR, resultResult.getOutput());
      evaluationContext.define(ATTRIBUTES_VAR, new TypedValue<>(resultResult.getAttributes(), DataType.builder()
          .mediaType(resultResult.getAttributesMediaType().orElse(MediaType.ANY)).build()));
    } else {
      TypedValue<String> body = ((TypedValue<String>) result);
      HttpResponseDWBinding responseDWBinding = new HttpResponseDWBinding(body, null);
      evaluationContext.define(DWBindings.RESPONSE.getBinding(), responseDWBinding);
      evaluationContext.define(PAYLOAD_VAR, result);
    }
  }

  private Result collectResultFromEvaluationContext(EvaluationContext evaluationContext) {
    TypedValue tOutput = getTypedValue(evaluationContext, PAYLOAD_VAR);
    TypedValue tOutputAttributes = getTypedValue(evaluationContext, ATTRIBUTES_VAR);

    Result.Builder builder = Result.builder();

    if (tOutputAttributes != null) {
      ((Optional) tOutputAttributes.getValue()).ifPresent(attributes -> builder.attributes(attributes));
      builder.attributesMediaType(tOutputAttributes.getDataType().getMediaType());
    }

    return builder
        .output(tOutput)
        .mediaType(tOutput.getDataType().getMediaType())
        .build();
  }

  private TypedValue getTypedValue(EvaluationContext evaluationContext, String variableName) {
    Object variable = evaluationContext.getVariables().get(variableName);
    if (variable == null) {
      return null;
    }
    if (!(variable instanceof TypedValue)) {
      throw new MuleRuntimeException(createStaticMessage(
                                                         format("'%s' variable from evaluation context should be a '%s', instead it is a '%s'. This is a probably a bug.",
                                                                variableName, TypedValue.class.getName(),
                                                                variable.getClass().getName())));
    }
    return (TypedValue) variable;
  }

}
